@@ -7,7 +7,7 @@ Inheriting the Agent class allows you to modify the behaviour of the agents in t
 from __future__ import annotations
 
 from copy import copy
-from typing import TYPE_CHECKING, Any, Optional, TypeVar
+from typing import TYPE_CHECKING, Any, Generator, Optional
 
 import pygame as pg
 from pygame.mask import Mask
@@ -22,6 +22,7 @@ from .proximity import ProximityIter
 from .util import random_angle, random_pos, round_pos
 
 if TYPE_CHECKING:
+    from ._static import _StaticSprite
     from .simulation import HeadlessSimulation, Shared
 
 
@@ -67,7 +68,16 @@ class Agent(Sprite):
     """The area in which the agent is free to move."""
 
     move: Vector2
-    """The current angle and speed used for the agent's movement.
+    """A two-dimensional vector representing the delta between its current and next position.
+
+    Note that `move` isn't added to the agent's `pos` automatically.
+    Instead, you should manually add the move delta to `pos`, like so:
+
+    >>> self.pos += self.move
+
+    The vector `Vector2(2, 1)` indicates that the agent will be moving 2 pixels along the x axis
+    and 1 pixel along the y axis.
+    You can use the `Vector2` class to calculate the agent's velocity by calling `length`.
     
     This property is also used to automatically rotate the agent's image
     when `vi.config.Schema.image_rotation` is enabled.
@@ -76,9 +86,6 @@ class Agent(Sprite):
     pos: Vector2
     """The current (centre) position of the agent."""
 
-    __previous_move: Optional[Vector2] = None
-    """The value of the `move` vector before the agent was frozen."""
-
     _obstacles: Group
     """The group of obstacles the agent can collide with."""
 
@@ -87,7 +94,8 @@ class Agent(Sprite):
 
     __simulation: HeadlessSimulation
 
-    _still_stuck: bool = False
+    _moving: bool = True
+    """The agent's movement will freeze when moving is set to False."""
 
     def __init__(
         self,
@@ -188,6 +196,59 @@ class Agent(Sprite):
 
         ...
 
+    def obstacle_intersections(
+        self, scale: float = 1
+    ) -> Generator[Vector2, None, None]:
+        """Retrieve the centre coordinates of all obstacle intersections.
+
+        If you not only want to check for obstacle collision,
+        but also want to retrieve the coordinates of pixel groups
+        that are colliding with your agent, then `obstacle_intersections` is for you!
+
+        If you only have one obstacle in your environment,
+        but it doesn't have a fill, only a stroke,
+        then your agent could possibly be colliding with different areas of the stroke.
+        Therefore, this method checks the bitmasks of both the agent and the obstacle
+        to calculate the overlapping bitmask.
+        From this overlapping bitmask,
+        the centre coordinates of all groups of connected pixels are returned.
+
+        To emulate a bigger (or smaller) radius,
+        you can pass along the `scale` option.
+        A scale of 2 makes your agent twice as big,
+        but only for calculating the intersecting bitmasks.
+        """
+
+        mask = pg.mask.from_surface(self.image)
+
+        # Scale the mask to the desired size
+        width, height = mask.get_size()
+        mask = mask.scale((width * scale, height * scale))
+
+        # Align the mask to the centre position of the agent
+        rect = mask.get_rect()
+        rect.center = self.center
+
+        for sprite in self._obstacles.sprites():
+            obstacle: _StaticSprite = sprite  # type: ignore
+
+            # Calculate the mask offset
+            x = obstacle.rect.x - rect.x
+            y = obstacle.rect.y - rect.y
+
+            overlap = mask.overlap_mask(obstacle.mask, offset=(x, y))
+
+            # For some reason PyGame has the wrong type hint here (single instead of list)
+            overlap_rects: list[pg.rect.Rect] = overlap.get_bounding_rects()  # type: ignore
+
+            for overlap_rect in overlap_rects:
+                # Undo the offset
+                overlap_rect.x += rect.x
+                overlap_rect.y += rect.y
+
+                # Return the centre coordinates of the connected pixels
+                yield Vector2(overlap_rect.center)
+
     def on_spawn(self):
         """Run any code when the agent is spawned into the simulation.
 
@@ -248,6 +309,9 @@ class Agent(Sprite):
         2. If the agent collides with any obstacles, then the agent will turn around 180 degrees.
         3. If the agent has not collided with any obstacles, it will have the opportunity to slightly change its angle.
         """
+        if not self._moving:
+            return
+
         changed = self.there_is_no_escape()
 
         prng = self.shared.prng_move
@@ -376,22 +440,50 @@ class Agent(Sprite):
         ...             self.freeze_movement()
         """
 
-        return bool(
-            pg.sprite.spritecollideany(self, self._sites, pg.sprite.collide_mask)  # type: ignore
-        )
+        return self.on_site_id() is not None
+
+    def on_site_id(self) -> Optional[int]:
+        """Get the identifier of the site the agent is currently on.
+
+        Examples
+        --------
+
+        Stop the agent's movement when it reaches a site to inspect.
+        In addition, the current site identifier is saved to the DataFrame.
+
+        ```python
+        class SiteInspector(Agent):
+            def update(self):
+                site_id = self.on_site_id()
+
+                # Save the site id to the DataFrame
+                self.save_data("site", site_id)
+
+                # bool(site_id) would be inaccurate
+                # as a site_id of 0 will return False.
+                # Therefore, we check whether it is not None instead.
+                if site_id is not None:
+                    # Inspect the site
+                    self.freeze_movement()
+        ```
+        """
+
+        site: Optional[_StaticSprite] = pg.sprite.spritecollideany(self, self._sites, pg.sprite.collide_mask)  # type: ignore
+
+        if site is not None:
+            return site.id
+        else:
+            return None
 
     def freeze_movement(self):
         """Freeze the movement of the agent. The movement can be continued by calling `continue_movement`."""
 
-        self.__previous_move = self.move
-        self.move = Vector2(0, 0)
+        self._moving = False
 
     def continue_movement(self):
-        """Continue the movement of the agent by using the angle and speed of the agent before its movement was frozen."""
+        """Continue the movement of the agent from before its movement was frozen."""
 
-        if self.__previous_move is not None:
-            self.move = self.__previous_move
-            self.__previous_move = None
+        self._moving = True
 
     def change_image(self, index: int):
         """Change the image of the agent.
